pub const OUTPUT_NAME: &str = "winit";
use std::{cell::RefCell, rc::Rc};

use smithay::{backend::{renderer::{ImportDma, ImportEgl}, winit}, reexports::wayland_server::Display, wayland::{dmabuf::init_dmabuf_global, output::Mode}};

mod state;
use crate::state::*;

#[cfg(feature = "winit")]
pub struct WinitData;

#[cfg(feature = "winit")]
impl Backend for WinitData {
    fn seat_name(&self) -> String {
        "winit".into()
    }
}


fn main() {
    println!("Hello, world!");
}

#[cfg(feature = "winit")]
fn run_winit() {
    let mut event_loop = smithay::reexports::calloop::EventLoop::try_new().unwrap();
    let display = Rc::new(RefCell::new(Display::new()));

    let (renderer, mut input) = match winit::init(None)  {
        Ok(ret) => ret,
        Err(_) => return
    };

    let renderer = Rc::new(RefCell::new(renderer));

    if renderer
        .borrow_mut()
        .renderer()
        .bind_wl_display(&display.borrow()).is_ok() {
            dbg!("EGL hardware-acceleration enabled");
        let dmabuf_formats = renderer
            .borrow_mut()
            .renderer()
            .dmabuf_formats()
            .cloned()
            .collect::<Vec<_>>();
        let renderer = renderer.clone();
        init_dmabuf_global(
            &mut *display.borrow_mut(),
            dmabuf_formats,
            move |buffer, _| renderer.borrow_mut().renderer().import_dmabuf(buffer).is_ok(),
            None,
        );
    }

    let size = renderer.borrow().window_size().physical_size;

    let data = WinitData{};

    let mut state = State::init(display.clone(), event_loop.handle(), data);

    let mode = Mode {
        size,
        refresh: 60_000
    };
}
